var express = require('express');
var fs = require('fs');
var xml2js = require('xml2js');
var Curso = require('../models/cursos.js');

var router = express.Router();


router.get('/nuevo/', function(req, res) {
  res.render('curso', {action: "/curso/crear/"});
});

router.post('/crear/', function(req, res) {
  var titulo = req.body.titulo;
  var numeroTemas = req.body.numeroTemas;

  var curso = new Curso({
    titulo:    titulo,
    numeroTemas: numeroTemas,
  });
  var parser = new xml2js.Parser();
  var salida;
  parser.addListener('end', function(result) {
    var arrayEstilosTitulos = obtenerEstilos(result);
    salida = obtenerTitulosImagenes(result, arrayEstilosTitulos);
  });

  for (var i = 0; i < numeroTemas; i++) {
    var tituloTema = req.body["titulo-tema"+(i+1)];
    var nombreVariableFichero = "fichero" + (i+1);
    var fichero = req.files[nombreVariableFichero];
    var tmpPath = fichero.path;
    var nombreActual = fichero.name;
    var data = fs.readFileSync(tmpPath, 'utf8');
    parser.parseString(data);
    var tema = {
      titulo:    tituloTema,
      fichero:  nombreActual,
      contenidos: []
    };
    var padre = 1;
    for(var j=0; j < salida.length; j++) {
      if(salida[j].hijo == 0) {
        var hijos = 1;
        var aux = {
          id: padre,
          tipo: salida[j].tipo,
          texto: salida[j].valor,
          comentario: "",
          hijo: salida[j].hijo
        };
        padre++;
      } else if(salida[j].hijo == 1){
        var aux = {
          id: hijos,
          tipo: salida[j].tipo,
          texto: salida[j].valor,
          comentario: "",
          hijo: salida[j].hijo
        };
        hijos++;
      } else {
        var aux = {
          id: -1,
          tipo: salida[j].tipo,
          texto: salida[j].valor,
          comentario: "",
          hijo: salida[j].hijo
        };
      }
      tema.contenidos.push(aux);
    };
    if(tema.contenidos.length != 0) {
      curso.temas.push(tema);
    }
  }

  if(curso.temas.length == 0) {
      res.json({status: 406});
  }else {
    curso.save(function(err) {
      if(!err) {
        console.log("Curso Creado");
        res.json({status: 201, data:curso._id});
      } else {
        console.log("Fallo al crear curso");
        console.log(err);
        res.json({status: 406, error: err.name});
      }
    });
  }
});

router.get('/id/:id', function(req, res) {
  return Curso.findById(req.params.id, function(err, curso) {
    if(curso) {
      var marcasDiapositivas  = "";
      for (var i = 0; i < curso.temas.length; i++) {
        var tema = curso.temas[i];
        var contTemas = "";
        if(tema.diapositivas) {
          for (var j = 0; j < tema.diapositivas.length; j++) {
            contTemas += tema.diapositivas[j].contenidos.length + ",";
          };
        }
        contTemas = contTemas.substring(0, contTemas.length-1);
        marcasDiapositivas += contTemas + ";";
      };

      marcasDiapositivas = marcasDiapositivas.substring(0, marcasDiapositivas.length-1);

      res.render('curso', {actualizar: 1, action: "/curso/actualizar/" ,curso:curso, marcasDiapositivas: marcasDiapositivas});
    } else {
      res.statusCode = 404;
      return res.send({ error: 'Not found' }); 
    }
  });
});

router.post('/eliminar/', function(req, res) {
  var idCurso = req.body.idCurso;
  return Curso.findById(idCurso, function(err, curso) {
    if(!curso) {
      res.json({status: 404, error: err.name});
    }

    return curso.remove(function(err) {
      if(!err) {
        console.log('Curso eliminado');
        return res.send({ status: 204 });
      } else {
        res.statusCode = 500;
        console.log('Internal error(%d): %s',res.statusCode,err.message);
        return res.send({ error: 'Server error' });
      }
    });
  });
});

router.post('/id/:id/message', function(req, res) {
  var idTema = req.body.idTema;
  var idContenido = req.body.idContenido;
  var comentario = req.body.comentario;
  var aux = 0;
  return Curso.findById(req.params.id, function(err, curso) {
    if(!curso) {
      res.json({status: 404, error: err.name});
    }
    for (var i = 0; i < curso.temas.length; i++) {
      if(curso.temas[i]._id == idTema) {
        for (var j = 0; j < curso.temas[i].contenidos.length; j++) {
          if(curso.temas[i].contenidos[j]._id == idContenido) {
            curso.temas[i].contenidos[j].comentario = comentario; 
            console.log("Se ha actualizado");
            aux = 1;
            break;
          }
        };
      }
      if(aux == 1) {
        break;
      }
    };
    return curso.save(function(err) {
      if(!err) {
        console.log('Updated');
        res.json({status: 200});
      } else {
        res.json({status: 400});
        console.log('Internal error(%d): %s',res.statusCode,err.message);
      }
    });
  });
});

router.post('/diapositiva/crear/', function(req, res) {
  var nDiapositivas = req.body.nDiapositivas;
  var idCurso = req.body.curso;
  var idTema = req.body.tema;
  var arrayDiapositivas = [];

  return Curso.findById(idCurso, function(err, curso) {
    if(!curso) {
      res.json({status: 404, error: err});
    }
    for (var i = 0; i < nDiapositivas; i++) {
      var diapositivas = {
        idCurso: idCurso,
        idTema: idTema,
        contenidos: []
      };
      var diapositiva = req.body['diapositiva'+i];
      var contenidos = diapositiva.split(';');
      for (var j = 0; j < contenidos.length; j++) {
        for (var k = 0; k < curso.temas.length; k++) {
          if(curso.temas[k]._id == idTema) {
            var posicionTema = k;
            for (var l = 0; l < curso.temas[k].contenidos.length; l++) {
              if(curso.temas[k].contenidos[l]._id == contenidos[j]) {
                var aux = {
                  id: curso.temas[k].contenidos[l].id,
                  tipo: curso.temas[k].contenidos[l].tipo, 
                  texto: curso.temas[k].contenidos[l].texto, 
                  comentario: curso.temas[k].contenidos[l].comentario, 
                  hijo: curso.temas[k].contenidos[l].hijo,
                };
                console.log(aux);
                diapositivas.contenidos.push(aux);
              }
            };
          }
        };
      };
      arrayDiapositivas.push(diapositivas);
    };
    curso.temas[posicionTema].diapositivas = arrayDiapositivas;

    return curso.save(function(err) {
      if(!err) {
        console.log('Updated');
        res.json({status: 200});
      } else {
        res.json({status: 400});
        console.log('Internal error(%d): %s',res.statusCode,err.message);
      }
    });
  });
});

router.get('/diapositivas/:idCurso/:idTema', function(req, res) {
  var idCurso = req.params.idCurso;
  var idTema = req.params.idTema;
  return Curso.findById(idCurso, function(err, curso) {
    if(!curso) {
      res.json({status: 404, error: err.name});
    }
    for (var i = 0; i < curso.temas.length; i++) {
      if(curso.temas[i]._id == idTema) {
        var tema = curso.temas[i];
        break;
      }
    }
    res.render('diapositiva', {tituloCurso: curso.titulo, tema: tema});
  });
});

function obtenerTitulosImagenes(result, arrayEstilosTitulos) {
  var parrafos = result['w:wordDocument']['w:body'][0]['w:p'];
  var salida = Array();
  for (var i = 0; i < parrafos.length; i++) {
    try {
      if(parrafos[i]['w:r'][0]['w:t']) {
        var estilo = obtenerEstilo(parrafos[i]);
        var esLista = obtenerLista(parrafos[i]);
        for (var j = 0; j < arrayEstilosTitulos.length; j++) {
          if(estilo) {
            if(arrayEstilosTitulos[j] == estilo) {
              if(esLista) {
                if(esLista == 1) {
                  salida.push({tipo: "titulo", hijo: 1, valor: parrafos[i]['w:r'][0]['w:t'][0] + ""});
                } else {
                  salida.push({tipo: "titulo", hijo: 0, valor: parrafos[i]['w:r'][0]['w:t'][0] + ""});
                }  
              } else {
                salida.push({tipo: "titulo", hijo: 0, valor: parrafos[i]['w:r'][0]['w:t'][0] + ""});  
              }
              break;
            }
          } else {
            break; 
          }
        };
      } else if(parrafos[i]['w:r'][0]['w:pict']) {
        for (var j = 0; j < arrayEstilosTitulos.length; j++) {
          if(arrayEstilosTitulos[j] == estilo) {
            estilo = null;
            salida.push({tipo: "imagen", hijo: -1, valor: parrafos[i]['w:r'][0]['w:pict'][0]['w:binData'][0]['_'] + ""});
            break;
          }
        }
      } 
    } catch(err) {
      console.log(err);
    }
  };

  return salida; 
};

function obtenerEstilo(parrafo) {
  try {
    return parrafo['w:pPr'][0]['w:pStyle'][0]['$']['w:val'];
  } catch(err) {
    return null;
  }
}

function obtenerLista(parrafo) {
  try {
    return parrafo['w:pPr'][0]['w:listPr'][0]['w:ilvl'][0]['$']['w:val'];
  } catch(err) {
    return null;
  }
}

function obtenerEstilos(result) {
  var estilos = result['w:wordDocument']['w:styles'][0]['w:style'];
  var estilosTitulos = Array();
  for (var i = 0; i < estilos.length; i++) {
    try {
      if(estilos[i]['$']['w:styleId'].indexOf("Heading")!= -1) {
        estilosTitulos.push(estilos[i]['$']['w:styleId']);
      } else {
        if(estilos[i]['w:basedOn'][0]['$']['w:val'].indexOf("Heading") != -1) {
          estilosTitulos.push(estilos[i]['$']['w:styleId']);
        }
      } 
    } catch(err) {
      console.log(err);
    }
  };
  return estilosTitulos;
};


module.exports = router;